'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var createComponent = require('reakit-system/createComponent');
var createHook = require('reakit-system/createHook');
require('../Box/Box.js');
var _rollupPluginBabelHelpers = require('../_rollupPluginBabelHelpers-b18c4e5e.js');
var React = require('react');
require('reakit-utils/useForkRef');
require('reakit-utils/isButton');
require('reakit-warning');
var useLiveRef = require('reakit-utils/useLiveRef');
require('reakit-utils/useIsomorphicEffect');
require('reakit-utils/hasFocusWithin');
require('reakit-utils/getActiveElement');
require('../Tabbable/Tabbable.js');
require('../Clickable/Clickable.js');
var Button = require('../Button/Button.js');
require('reakit-utils/useSealedState');
var getDocument = require('reakit-utils/getDocument');
require('../Id/IdProvider.js');
require('../Id/IdState.js');
require('reakit-utils/useUpdateEffect');
require('reakit-utils/isPromise');
require('reakit-utils/isEmpty');
require('reakit-utils/isPlainObject');
var FormState = require('./FormState.js');
require('reakit-utils/isObject');
require('./utils/setAllIn.js');
require('./utils/getIn.js');
require('reakit-utils/toArray');
require('reakit-utils/isInteger');
require('./utils/setIn.js');

function getFirstInvalidInput(baseId, target) {
  var document = getDocument.getDocument(target);
  var selector = "[aria-invalid=true][id^=" + baseId + "]";
  return document.querySelector(selector);
}

var unstable_useFormSubmitButton = createHook.createHook({
  name: "FormSubmitButton",
  compose: Button.useButton,
  useState: FormState.unstable_useFormState,
  useOptions: function useOptions(options) {
    return _rollupPluginBabelHelpers._objectSpread2({
      disabled: options.submitting
    }, options);
  },
  useProps: function useProps(options, _ref) {
    var htmlOnClick = _ref.onClick,
        htmlProps = _rollupPluginBabelHelpers._objectWithoutPropertiesLoose(_ref, ["onClick"]);

    var onClickRef = useLiveRef.useLiveRef(htmlOnClick);
    var onClick = React.useCallback(function (event) {
      var _onClickRef$current;

      (_onClickRef$current = onClickRef.current) === null || _onClickRef$current === void 0 ? void 0 : _onClickRef$current.call(onClickRef, event);
      if (event.defaultPrevented) return;
      var self = event.currentTarget;
      window.requestAnimationFrame(function () {
        var input = getFirstInvalidInput(options.baseId, self);
        input === null || input === void 0 ? void 0 : input.focus();

        if (input && "select" in input) {
          input.select();
        }
      });
    }, [options.baseId]);
    return _rollupPluginBabelHelpers._objectSpread2({
      type: "submit",
      onClick: onClick
    }, htmlProps);
  }
});
var unstable_FormSubmitButton = createComponent.createComponent({
  as: "button",
  memo: true,
  useHook: unstable_useFormSubmitButton
});

exports.unstable_FormSubmitButton = unstable_FormSubmitButton;
exports.unstable_useFormSubmitButton = unstable_useFormSubmitButton;
